@@ -559,9 +559,8 @@ public:
 
         const bm::word_t* const* blk_blk = top_blocks_[i];
         if ((bm::word_t*)blk_blk == FULL_BLOCK_FAKE_ADDR)
-            return FULL_BLOCK_FAKE_ADDR;
-        const bm::word_t* ret = (blk_blk == 0) ? 0 : blk_blk[j];
-        return ret;
+            return (bm::word_t*)blk_blk;
+        return (blk_blk) ? blk_blk[j] : 0;
     }
     /**
         \brief Finds block in 2-level blocks array (unsinitized)
@@ -575,9 +574,8 @@ public:
             return 0;
         bm::word_t* const* blk_blk = top_blocks_[i];
         if ((bm::word_t*)blk_blk == FULL_BLOCK_FAKE_ADDR)
-            return FULL_BLOCK_FAKE_ADDR;
-        bm::word_t* ret = (!blk_blk) ? 0 : blk_blk[j];
-        return ret;
+            return (bm::word_t*)blk_blk;
+        return (blk_blk) ? blk_blk[j] : 0;
     }
 
 
@@ -861,7 +859,7 @@ public:
         return new_block;
     }
     
-    /** Clone static known block, assign to i-j position
+    /** Clone static known block, assign to i-j position. Old block must be NULL.
         @return new block address
     */
     void clone_gap_block(unsigned  i,
@@ -869,10 +867,18 @@ public:
                          const bm::gap_word_t* gap_block,
                          unsigned len)
     {
+        BM_ASSERT(get_block(i, j) == 0);
+
         int new_level = bm::gap_calc_level(len, this->glen());
+        if (!new_level) // level 0, check if it is empty
+        {
+            if (bm::gap_is_all_zero(gap_block))
+                return;
+        }
+
         if (new_level < 0)
         {
-            convert_gap2bitset(i, j, gap_block);
+            convert_gap2bitset(i, j, gap_block, len);
             return;
         }
         bm::gap_word_t* new_blk = allocate_gap_block(unsigned(new_level), gap_block);
@@ -1502,10 +1508,12 @@ public:
         \param i - top index.
         \param j - secondary index.
         \param gap_block - Pointer to the gap block, if NULL block [i,j] is taken
+        \param len - gap length
         \return new bit block's memory
     */
     bm::word_t* convert_gap2bitset(unsigned i, unsigned j,
-                                   const gap_word_t* gap_block=0)
+                                   const gap_word_t* gap_block=0,
+                                   unsigned len = 0)
     {
         BM_ASSERT(is_init());
         
@@ -1517,7 +1525,7 @@ public:
         BM_ASSERT(IS_VALID_ADDR((bm::word_t*)gap_block));
 
         bm::word_t* new_block = alloc_.alloc_bit_block();
-        bm::gap_convert_to_bitset(new_block, gap_block);
+        bm::gap_convert_to_bitset(new_block, gap_block, len);
         
         top_blocks_[i][j] = new_block;
 
@@ -1558,6 +1566,13 @@ public:
             set_block_ptr(i, j, new_block);
             return new_block;
         }
+        return deoptimize_block_no_check(block, i, j);
+    }
+
+    bm::word_t* deoptimize_block_no_check(bm::word_t* block,
+                                         unsigned i, unsigned j)
+    {
+        BM_ASSERT(block);
         if (BM_IS_GAP(block))
         {
             gap_word_t* gap_block = BMGAP_PTR(block);
@@ -1582,6 +1597,7 @@ public:
     }
 
 
+
     /**
         Free block, make it zero pointer in the tree
     */
